import es, cmdlib, playerlib, popuplib
import os, random

info = es.AddonInfo()
info['name']        = "EasySpawn"
info['version']     = "1.0.0"
info['author']      = "DanielB" 
info['url']         = "http://forums.eventscripts.com/viewtopic.php?t=34141" 
info['description'] = "Easy spawnpoint creator for DanielBs Advanced Deathmatch" 

dm = es.import_addon('dm')

spawn_path = dm.addondir + '/dm/addons/easyspawn/maps/'

spawn_points = []

def load():
    #cmdlib.registerSayCommand('!spawnpoint', cmd_add, 'Create a spawnpoint at your current location')
    cmdlib.registerSayCommand('!spawnpoint', cmd_admin, 'Spawnpoint admin')
    
    if not os.path.isdir(spawn_path):
        os.makedirs(spawn_path)
    
    loadLocations()
    
def unload():
    cmdlib.unregisterSayCommand('!spawnpoint')
    
def checkPop(pname):
    if popuplib.exists(pname):
        popuplib.delete(pname)
        
def cmd_admin(userid, args):
    if dm.isAuthed(userid):
        p_admin_main.send(userid)
        
def select_main(userid, choice, popupname):
    if choice == 'new':
        cmd_add(userid, [])
        p_admin_main.send(userid)
    elif choice == 'next':
        if spawn_points:
            generate_menu(userid, -1).send(userid)
        else:
            dm.tell(userid, 'You must make a spawn point first')
            p_admin_main.send(userid)
        
        
def teleport_to(userid, index):
    #es.dbgmsg(0, index)
    location, angle = spawn_points[index]
    x, y, z = location.split(',')
    es.server.queuecmd('es_xsetpos %s %s %s %s'%(userid, x, y, z))
        
def generate_menu(userid, index=-1):
    if index >= len(spawn_points)-1 or index == -1:
        index = 0
    else:
        index += 1

    pname = 'dm_admin_spawnmanage_%s'%userid
    checkPop(pname)
    p = popuplib.easymenu(pname, None, select_manage)
    p.settitle('Manage Spawnpoint')
    p.addoption((index, 'delete'), 'Delete Spawn Point')
    p.addoption((index, 'next'), 'Next')
    teleport_to(userid, index)
    return p
    
def select_manage(userid, choice, popupname):
    checkPop(popupname)
    index, selection = choice
    
    if selection == 'next':
        if spawn_points:
            generate_menu(userid, index).send(userid)
        else:
            dm.tell(userid, 'You must make a spawn point first')
            p_admin_main.send(userid)
        
    elif selection == 'delete':
        delete_spawn(index)
        dm.tell(userid, 'Deleted spawn point')
        generate_menu(userid, index-1).send(userid)
    
def delete_spawn(index):
    spawn_points.pop(index)
    
    cmap = str(es.ServerVar('eventscripts_currentmap'))
    filename = spawn_path + cmap + '.txt'
    if not os.path.isfile(filename):
        open(filename, 'w').close()
        
    for loc, ang in spawn_points:
        x = open(filename, 'w')
        x.write('%s %s\n'%(loc, ang))
        x.close()
        
        
def cmd_add(userid, args):
    if dm.isAuthed(userid):
        location = reduce(lambda x,y: '%s,%s'%(x,y), es.getplayerlocation(userid))
        angle = reduce(lambda x,y: '%s,%s'%(x,y), playerlib.getPlayer(userid).getViewAngle())
        
        #spawn_locations[type][team].append([location,angle])
        create_spawn(location,angle)
        
        cmap = str(es.ServerVar('eventscripts_currentmap'))
        filename = spawn_path + cmap + '.txt'
        if not os.path.isfile(filename):
            open(filename, 'w').close()
        
        x = open(filename, 'a')
        x.write('%s %s\n'%(location, angle))
        x.close()
        spawn_points.append((location, angle))
        
        dm.tell(userid, 'Created Spawn Point [%s] [%s]'%(location,angle))
    else:
        dm.tell(userid, 'You don\'t have access to that command!')
    
def es_map_start(ev):
    loadLocations()
    
def loadLocations():
    global spawn_points
    cmap = str(es.ServerVar('eventscripts_currentmap'))
    
    while spawn_points:
        spawn_points.pop(0)
    
    if not cmap: 
        return
    
    filename = spawn_path + cmap + '.txt'
    if os.path.isfile(filename):
        userid = es.getuserid()
        
        killSpawns(userid)
        
        spawnfile = open(filename, 'r')
        lines = spawnfile.readlines()
        spawnfile.close()
        for line in lines:
            line = line.strip()
            if line:
                loc, ang = line.split(' ')
                create_spawn(loc, ang, userid)
                spawn_points.append((loc,ang))
                
def create_spawn(loc, ang, userid=None):
    if not userid:
        userid = es.getuserid()
    if userid:
        es.server.queuecmd(execstr%(userid, 'info_player_counterterrorist', loc, ang))
        es.server.queuecmd(execstr%(userid, 'info_player_terrorist', loc, ang))
        
            
def killSpawns(userid=None):
    if not userid:
        userid = es.getuserid()
    
    if userid:
        for index in getSpawnIndexs():
            es.server.queuecmd('es_xremove %s'%index)
            
def getSpawnIndexs():
    entlist = lambda x: list(es.createentitylist(x))
    return entlist('info_player_terrorist') + entlist('info_player_counterterrorist')
    
    
execstr = 'es_xgive %s %s;es_setindexprop server_var(eventscripts_lastgive) CBaseEntity.m_vecOrigin %s;es_setindexprop server_var(eventscripts_lastgive) CBaseEntity.m_angRotation %s'


p_admin_main = popuplib.easymenu('dm_admin_main', None, select_main)
p_admin_main.settitle('Spawn Points Menu')
p_admin_main.addoption('new', 'Make Spawnpoint')
p_admin_main.addoption('next', 'Move to next spawnpoint')

